# This file is part of 'Cut The Crap (CTC)'
# Copyright 1999-2000 Constantinos A. Kotsokalis <ckotso@cs.ntua.gr>
#
# CTC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# CTC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CTC; see the file COPYING. If not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

import sys, string, os, os.path
import zaplet_engine, name_services, acl, noproxy

conf_dict = {
	"RETURN_GIF"        : "self.return_gif = self.make_return_gif(tokens[1:])",
	"CTC_GIF"           : "self.ctc_gif = self.make_ctc_gif(tokens[1:])",
	"BIND_PORT"         : "self.bind_port = self.make_bind_port(tokens[1:])",
	"ZAPLET_DIR"        : "self.zaplet_engine = self.make_zaplet_engine(tokens[1:])",
	"ERROR_HTML_FILE"   : "self.error_html_file = self.make_error_html_file(tokens[1:])",
	"PARENT_PROXY_HOST" : "self.parent_proxy_host = self.make_parent_proxy_host(tokens[1:])",
	"PARENT_PROXY_PORT" : "self.parent_proxy_port = self.make_parent_proxy_port(tokens[1:])",
	"NOPROXY"           : "self.noproxy = self.make_noproxy(tokens[1:])",
	"DNS_CHECK_SEC"     : "self.dns_check_sec = self.make_dns_check_sec(tokens[1:])",
	"NAME_CACHE"        : "self.name_cache = self.make_name_cache(tokens[1:])",
	"ACL"               : "self.acl = self.make_acl(tokens[1:])",
	"NO_LOGGING"        : "self.no_logging = self.make_no_logging(tokens[1:])",
	"PLUGIN_DIR"        : "self.plugins, self.states = self.make_plugins(tokens[1:])"
}

class config:

	def __init__(self, cf, errobj):
		self.errobj = errobj
		try:
			fp = open(cf, "r")
		except IOError, msg:
			errobj.eprint_exit("Config file could not be opened: ", msg)

		lines = fp.readlines()
		fp.close()

		self.return_gif = ''
		self.ctc_gif = ''
		self.bind_port = 5050
		self.zaplet_dir = ''
		self.error_html_file = ''
		self.parent_proxy_host = ''
		self.parent_proxy_ip = ''
		self.parent_proxy_port = -1
		self.noproxy = noproxy.noproxy(errobj)
		self.dns_check_sec = 1800
		self.name_cache = name_services.dnscache('', errobj, self)
		self.acl = acl.acl(errobj)
		self.no_logging = 0
		self.plugins = []
		self.states = []

		self.read_conf(lines)


	def read_conf(self, lines):
		for line in lines:
			line = string.strip(line)
			if not len(line) or line[0] == '#':
				continue
			tokens = string.split(line)
			cmd = tokens[0]
			exec(conf_dict[cmd])


	def make_return_gif(self, tokens):
		try:
			fp = open(tokens[0], "r")
		except IOError, msg:
			self.errobj.eprint_exit("Cannot open %s: "%(tokens[0]), msg)
		data = fp.read()
		fp.close()
		return """HTTP/1.0 200 OK
Content-type: image/gif
Content-length: """ + `len(data)` + "\r\n\r\n" + data


	def make_ctc_gif(self, tokens):
		return self.make_return_gif(tokens)


	def make_bind_port(self, tokens):
		return string.atoi(tokens[0])


	def make_zaplet_engine(self, tokens):
		return zaplet_engine.zaplet_engine(tokens[0])


	def make_error_html_file(self, tokens):
		try:
			fp = open(tokens[0], "r")
		except IOError, msg:
			self.errobj.eprint_exit("Cannot open %s: "%(tokens[0]), msg)

		data = fp.read()
		fp.close()
		return data


	def make_parent_proxy_host(self, tokens):
		self.parent_proxy_ip = self.name_cache.name_check(tokens[0])
		return tokens[0]


	def make_parent_proxy_port(self, tokens):
		return string.atoi(tokens[0])


	def make_dns_check_sec(self, tokens):
		return string.atoi(tokens[0])


	def make_name_cache(self, tokens):
		return name_services.dns_cache(tokens[0], self.errobj)


	def make_no_logging(self, tokens):
		return string.atoi(tokens[0])


	def make_acl(self, tokens):
		if len(tokens) < 2 or (tokens[0] not in ["ALLOW","DENY"]):
			self.errobj.eprint_exit("Invalid ACL directive", '')
		tacl = self.acl
		if tokens[0] == "ALLOW":
			if tokens[1] == "ALL":
				tacl.all_allow()
			else:
				tacl.add_allow(tokens[1])
		elif tokens[0] == "DENY":
			if tokens[1] == "ALL":
				tacl.all_deny()
			else:
				tacl.add_deny(tokens[1])
		return tacl


	def make_noproxy(self, tokens):
		np = self.noproxy
		np.add(tokens[0])
		return np


	def make_plugins(self, tokens):
		plugin_dir = tokens[0]
		plugins = []
		states = []
		files = os.listdir(plugin_dir)
		for plugin in files:
			filename = os.path.join(plugin_dir, plugin)
			if not os.path.isfile(filename):
				continue
			try:
				fp = open(filename)
			except IOError, msg:
				self.errobj.eprint_exit("Cannot open " + plugin + ": ", msg)
			plugins.append(compile(fp.read(), filename, 'exec'))
			states.append(None)
			fp.close()
		return plugins, states


